/*****************************
   Author: Thomas Schmidt, UAS Kaiserslautern, Working Group "Smart Machines"
   Goal: prototype (proof of concept)
   State: code running
   Date: 09.09.2018
   
   Description:
   This sketch is used in combination with a T-Skin device, the Crazyflie 2.0 quadcopter, a Crazyradio USB dongle and the Crazyflie client.
   The sketch is uploaded to the T-Skin and constantly updates an array with 7 unsigned chars with accData values and sends it via BLE. The Crazyflie client
   then reads those values and maps them to input parameters for the Crazyflie 2.0 quadcopter. 
  
   2 values per axis are needed because unsigned char is unable to encode negative values, in order to compensate this
   an additional value encodes the sign of each axis. Furthermore this sketch uses one of the T-Skin's hardware buttons to
   activate/deactivate the Crazyflie's hover mode.
   
   Architecture:
   - accData values are used instead of roll pitch and yaw angles in order to map the T-Skins inclination to control parameters of the Crazyflie client

   Bugs/To be done:
   - Due to the fact that acceleration data is used for sending the x, y and z angles of the T-Skin, fast movements in any directions result in sending
   those values via BLE. This behaviour is unwanted and needs to be fixed.
*/


#include <tactigon_led.h>
#include <tactigon_IMU.h>
#include <tactigon_BLE.h>
#include <tactigon_IO.h>
#include <tactigon_Env.h>


T_Led rLed, bLed, gLed;
T_ACC accMeter;
T_AccData accData;
T_GPP gpp1;

T_BLE bleManager;
UUID uuid;
T_BLE_Characteristic bleChar;
int ticks;
//Used for LEDs cycle
int debounceTime = 300;
int lastButtonPress;
int  ticksLed, stp, cnt;
float threshold = 2.0;
boolean heightmode = false;

void setup() {
  
  ticks = 0;
  ticksLed = 0;
  stp = 0;
  cnt = 0;

  rLed.init(T_Led::RED);
  gLed.init(T_Led::GREEN);
  bLed.init(T_Led::BLUE);

  rLed.off();
  gLed.off();
  bLed.off();
  bleManager.InitRole(TACTIGON_BLE_PERIPHERAL);
  //bleManager.setName("TTGPS");
  //add acc characteristic
  uuid.set("7ac71000-503d-4920-b000-acc000000001");
  bleChar = bleManager.addNewChar(uuid, 7);
  lastButtonPress = 0;
  //GPIO initialization
  gpp1.init(T_GPP::GPP1, T_GPP::GPP_IN);
}

void flash() {
  rLed.on();
  gLed.on();
  bLed.on();
}

void loop() {
  // array for 7 unsigned char values
  unsigned char buffData[7] = {};
  //check sensors at 50Hz
  if (GetCurrentMilli() >= (ticksLed + (1000 / 50))) {
    ticks = GetCurrentMilli();
    //get acc data
    accData = accMeter.getAxis();

    //read hardware button and set heightmode accordingly
    if(!gpp1.read() && heightmode == false) {
      if (GetCurrentMilli()>= lastButtonPress + debounceTime){
        heightmode = true;
        lastButtonPress = GetCurrentMilli();
      }
    }

    if(!gpp1.read() && heightmode == true) {
      if (GetCurrentMilli()>= lastButtonPress + debounceTime){
        heightmode = false;
        lastButtonPress = GetCurrentMilli();
      }
    }
    
    /**	read x,y and z accData and write the data to unsigned char array. If the values are negative,
    	the corresponding array indices are set to '0'. This way the Crazyflie client can handle the x, y and z 
    	inclination accordingly. Threshold prevents a steering that is too sensitive to unwanted movement of the user's hand.
    */
    if (accData.y < 0 && (fabs)((float)accData.y) > threshold) {
      buffData[0] = 1;
      buffData[1] = (accData.y * 5) * (-1);
    }

    if (accData.y > 0 && (fabs)((float)accData.y) > threshold) {
      buffData[0] = 0;
      buffData[1] = accData.y * 5;
    }
    
    if (accData.x < 0 && (fabs)((float)accData.x) > threshold) {
      buffData[2] = 1;
      buffData[3] = (accData.x * 5) * (-1);
    }

    if (accData.x > 0 && (fabs)((float)accData.x) > threshold) {
      buffData[2] = 0;
      buffData[3] = accData.x * 5;
    }
    
    if (accData.z < 0 && (fabs)((float)accData.z) > threshold) {
      buffData[4] = 1;
      buffData[5] = (accData.z * 10) * (-1);    
    }

    if (accData.z > 0 && (fabs)((float)accData.z) > threshold) {
      buffData[4] = 0;
      buffData[5] = accData.z * 10;
    }

    if(heightmode == false){
      buffData[6] = 0;
    }
    if(heightmode == true){
      buffData[6] = 1;
    }
  }
}
